#
# CMFExtFile tests
#

from Products.CMFExtFile import testing
from Products.CMFCore.permissions import ModifyPortalContent

notImage = testing.notImage
makeFileUpload = testing.makeFileUpload

default_user = testing.default_user
default_password = testing.default_password
auth_info = '%s:%s' % (default_user, default_password)

from Products.CMFExtFile.config import manage_titles


class TestExtFileAdd(testing.FunctionalTestCase):

    def testAddFileFromString(self):
        # Add ExtFile from string
        file = open(notImage, 'rb').read()
        self.folder.invokeFactory('ExtFile', id='file', file=file)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))

    def testAddFileFromFileHandle(self):
        # Add ExtFile from file handle
        file = open(notImage, 'rb')
        self.folder.invokeFactory('ExtFile', id='file', file=file)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))

    def testAddFileFromFileUpload(self):
        # Add ExtFile from file upload
        file = makeFileUpload(notImage, 'application/zip')
        self.folder.invokeFactory('ExtFile', id='file', file=file)
        self.assertEqual(self.folder.file.Format(), 'application/zip')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))

    def testAddFileFromStringWithFormat(self):
        # Add ExtFile from file string passing a content_type
        file = open(notImage, 'rb').read()
        self.folder.invokeFactory('ExtFile', id='file', file=file, format='application/zip')
        self.assertEqual(self.folder.file.Format(), 'application/zip')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))

    def testAddFileFromFileHandleWithFormat(self):
        # Add ExtFile from file handle passing a content_type
        file = open(notImage, 'rb')
        self.folder.invokeFactory('ExtFile', id='file', file=file, format='application/zip')
        self.assertEqual(self.folder.file.Format(), 'application/zip')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))

    def testAddFileWithIdFromFilename(self):
        # Add ExtFile without id - should use filename instead
        file = makeFileUpload(notImage, 'application/zip', filename='foo.zip')
        self.folder.invokeFactory('ExtFile', id='', file=file)
        self.assertEqual(self.folder['foo.zip'].Format(), 'application/zip')
        self.assertEqual(self.folder['foo.zip'].get_size(), self._fsize(notImage))

    def testPUTAddFile(self):
        # FTP PUT an ExtFile
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/file',
                                stdin=open(notImage, 'rb'),
                                env={'CONTENT_TYPE': 'application/octet-stream'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 201)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))


class TestExtFileEdit(testing.FunctionalTestCase):

    def afterSetUp(self):
        testing.FunctionalTestCase.afterSetUp(self)
        self.folder.invokeFactory('ExtFile', id='file')
        self.assertEqual(self.folder.file.Format(), '')
        self.assertEqual(self.folder.file.get_size(), 0)

    def testEditFileFromString(self):
        # Edit file passing string
        file = open(notImage, 'rb').read()
        self.folder.file.edit(file=file)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))

    def testEditFileFromFileHandle(self):
        # Edit file passing file handle
        file = open(notImage, 'rb')
        self.folder.file.edit(file=file)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))

    def testEditFileFromFileUpload(self):
        # Edit file passing FileUpload instance
        # NB: Must provide filename here or FileUpload evaluates to false
        file = makeFileUpload(notImage, 'application/zip', filename='foo')
        self.folder.file.edit(file=file)
        self.assertEqual(self.folder.file.Format(), 'application/zip')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))

    def testPUTEditFile(self):
        # FTP PUT an ExtFile
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/file',
                                stdin=open(notImage, 'rb'),
                                env={'CONTENT_TYPE': 'application/zip'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 204)
        self.assertEqual(self.folder.file.Format(), 'application/zip')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))


class TestExtFileCataloging(testing.FunctionalTestCase):

    def afterSetUp(self):
        testing.FunctionalTestCase.afterSetUp(self)
        self.catalog = self.portal.portal_catalog
        self.catalog.addColumn('Format')
        self.workflow = self.portal.portal_workflow
        self.workflow.setChainForPortalTypes(('ExtFile',), ())

    def testAddFileFromFileString(self):
        # Add ExtFile from string
        self.folder.invokeFactory('ExtFile', id='file', title='Foo',
                                  file=open(notImage, 'rb').read())
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/octet-stream')
        self.assertEqual(brain.Title, 'Foo')

    def testAddFileFromFileHandle(self):
        # Add ExtFile from file handle
        self.folder.invokeFactory('ExtFile', id='file', title='Foo',
                                  file=open(notImage, 'rb'))
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/octet-stream')
        self.assertEqual(brain.Title, 'Foo')

    def testAddFileFromFileUpload(self):
        # Add ExtFile from file upload
        self.folder.invokeFactory('ExtFile', id='file', title='Foo',
                                  file=makeFileUpload(notImage, 'application/zip'))
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/zip')
        self.assertEqual(brain.Title, 'Foo')

    def testAddFileFromFileHandleWithFormat(self):
        # Add ExtFile from file handle passing a content_type
        self.folder.invokeFactory('ExtFile', id='file', title='Foo',
                                  file=open(notImage, 'rb'), format='application/zip')
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/zip')
        self.assertEqual(brain.Title, 'Foo')

    def testAddFileWithIdFromFilename(self):
        # Add ExtFile without id - should use filename instead
        self.folder.invokeFactory('ExtFile', id='', title='Foo',
                                  file=makeFileUpload(notImage, 'application/zip', filename='foo.zip'))
        brain = self.catalog(getId='foo.zip')[0]
        self.assertEqual(brain.Format, 'application/zip')
        self.assertEqual(brain.Title, 'Foo')

    def testPUTAddFile(self):
        # FTP PUT an ExtFile
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/file',
                                stdin=open(notImage, 'rb'),
                                env={'CONTENT_TYPE': 'application/zip'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 201)
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/zip')
        if manage_titles(self.folder):
            self.assertEqual(brain.Title, 'file')
        else:
            self.assertEqual(brain.Title, '')

    def testEditFileFromString(self):
        # Edit file passing string
        self.folder.invokeFactory('ExtFile', id='file')
        self.folder.file.edit(file=open(notImage, 'rb').read())
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/octet-stream')

    def testEditFileFromFileHandle(self):
        # Edit file passing file handle
        self.folder.invokeFactory('ExtFile', id='file')
        self.folder.file.edit(file=open(notImage, 'rb'))
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/octet-stream')

    def testEditFileFromFileUpload(self):
        # Edit file passing FileUpload instance
        self.folder.invokeFactory('ExtFile', id='file')
        # NB: Must provide filename here or FileUpload evaluates to false
        self.folder.file.edit(file=makeFileUpload(notImage, 'application/zip', filename='foo'))
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/zip')

    def testPUTEditFile(self):
        # FTP PUT an ExtFile
        self.folder.invokeFactory('ExtFile', id='file')

        # We don't have workflow here so we do this by hand
        self.folder.file.manage_permission(ModifyPortalContent, ('Manager', 'Owner'))

        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/file',
                                stdin=open(notImage, 'rb'),
                                env={'CONTENT_TYPE': 'application/zip'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 204)
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/zip')
        if manage_titles(self.folder):
            self.assertEqual(brain.Title, 'file')
        else:
            self.assertEqual(brain.Title, '')


class TestExtFilePublishing(testing.FunctionalTestCase):

    def testPublish(self):
        self.folder.invokeFactory('ExtFile', id='file', file=open(notImage, 'rb'), format='application/zip')
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/file', basic=auth_info)
        self.assertEqual(response.getStatus(), 200)
        self.failUnless(response.getHeader('Content-Type').startswith('application/zip'))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestExtFileAdd))
    suite.addTest(makeSuite(TestExtFileEdit))
    suite.addTest(makeSuite(TestExtFileCataloging))
    suite.addTest(makeSuite(TestExtFilePublishing))
    return suite

